"""
Model Context Protocol (MCP) module for reactive-ai-agent framework.
Provides client implementation and helper functions for MCP.
"""

from .client import MCPClient

__all__ = [
    "MCPClient",
]
